@@ -3,8 +3,8 @@ a.est <- function(x) {
    runtime <- proc.time()
    opt <- optimize( a.mle, c(-1, 1), x = x, maximum = TRUE )
    best <- opt$maximum
-   mod <- alpha.mle(x, best)
+   mod <- Compositional::alpha.mle(x, best)
    runtime <- proc.time() - runtime
-   list(runtime = runtime, best = best, loglik = mod$loglik, 
+   list(runtime = runtime, best = best, loglik = mod$loglik,
         p = mod$p, mu = mod$mu, su = mod$su)
-}
\ No newline at end of file
+}
