@@ -7,7 +7,7 @@
 #### Regression analysis with compositional data containing zero values
 #### Chilean Journal of Statistics, 6(2): 47-57
 ################################
-alfareg.tune <- function(y, x, a = seq(0.1, 1, by = 0.1), nfolds = 10, folds = NULL, nc = 1,
+alfareg.tune <- function(y, x, a = seq(0.1, 1, by = 0.1), nfolds = 10, folds = NULL, ncores = 1,
                          seed = NULL, graph = FALSE) {
   ## y is the compositional data (dependent variable)
   ## x is the independent variables
@@ -19,11 +19,9 @@ alfareg.tune <- function(y, x, a = seq(0.1, 1, by = 0.1), nfolds = 10, folds = N
   n <- dim(y)[1]
   ina <- 1:n
   x <- model.matrix(y ~., data.frame(x) )
-  if ( is.null(folds) )  folds <- Compositional::makefolds(ina, nfolds = nfolds,
-                                                           stratified = FALSE, seed = seed)
+  if ( is.null(folds) )  folds <- Compositional::makefolds(ina, nfolds = nfolds, stratified = FALSE, seed = seed)
   nfolds <- length(folds)
-
-  if (nc <= 1) {
+  if ( ncores <= 1 ) {
     apa <- proc.time()
     kula <- matrix(nrow = nfolds, ncol = la)
     for (j in 1:la) {
@@ -38,30 +36,34 @@ alfareg.tune <- function(y, x, a = seq(0.1, 1, by = 0.1), nfolds = 10, folds = N
         kula[i, j] <- sum(yu * log(yu / yest), na.rm = TRUE)
       }
     }
-
     kl <- Rfast::colmeans(kula)
     opt <- a[ which.min(kl) ]
     val <- which.min(kl)
     per <- min(kl, na.rm = TRUE)
     pera <- Rfast::rowMins(kula, value = TRUE)  ## apply(kula, 1, min)
     apa <- proc.time() - apa
-
   } else {
     apa <- proc.time()
-    val <- matrix(a, ncol = nc) ## if the length of a is not equal to the
+    val <- matrix(a, ncol = ncores) ## if the length of a is not equal to the
     ## dimensions of the matrix val a warning message should appear
-    requireNamespace("doParallel", quietly = TRUE, warn.conflicts = FALSE)
-    cl <- parallel::makePSOCKcluster(nc)
-    doParallel::registerDoParallel(cl)
-    if ( is.null(folds) )  folds <- Compositional::makefolds(ina, nfolds = nfolds,
-                                                             stratified = FALSE, seed = seed)
-    kula <- foreach::foreach(j = 1:nc, .combine = cbind, .packages = "Rfast", .export = c("alfa.reg",
-	        "alfa", "helm", "comp.reg", "multivreg", "rowsums", "colmeans", "colVars") ) %dopar% {
+    
+    cl <- parallel::makeCluster(ncores)
+    # Load required packages on workers
+    parallel::clusterEvalQ(cl, {
+      library(Compositional)
+      library(Rfast)
+    })
+    # Export only what workers need
+    parallel::clusterExport(cl, 
+                           varlist = c("y", "x", "folds", "nfolds", "val"), 
+                           envir = environment())
+    
+    kula <- parallel::parSapply(cl, 1:ncores, function(j) {
        ba <- val[, j]
        ww <- matrix(nrow = nfolds, ncol = length(ba) )
        for ( l in 1:length(ba) ) {
           ytr <- Compositional::alfa(y, ba[l])$aff
-          for (i in 1:nfolds) {
+          for ( i in 1:nfolds ) {
             xu <- x[ folds[[ i ]], -1 , drop = FALSE]
             yu <- y[ folds[[ i ]], ]
             xa <- x[ -folds[[ i ]], -1]
@@ -71,11 +73,11 @@ alfareg.tune <- function(y, x, a = seq(0.1, 1, by = 0.1), nfolds = 10, folds = N
             ww[i, l] <- sum(yu * log(yu / yest), na.rm = TRUE)
           }
        }
-       return(ww)
-    }
-    parallel::stopCluster(cl)
-
-    kula <- kula[, 1:la]
+       ww 
+    })
+    
+    parallel::stopCluster(cl)    
+    kula <- matrix(kula, nrow = nfolds)[, 1:la]
     kl <- Rfast::colmeans(kula)
     opt <- a[ which.min(kl) ]
     val <- which.min(kl)
@@ -83,7 +85,6 @@ alfareg.tune <- function(y, x, a = seq(0.1, 1, by = 0.1), nfolds = 10, folds = N
     pera <- Rfast::rowMins(kula, value = TRUE)   ## apply(kula, 1, min)
     apa <- proc.time() - apa
   }
-
   if ( graph ) {
     plot( a, kl, type = 'b', ylim = c( min(kl), max(kl) ), xlab = expression(alpha),
 	      ylab = '2 * Kullback Leibler divergence', cex.lab = 1.2, cex.axis = 1.2, pch = 16,
@@ -91,6 +92,5 @@ alfareg.tune <- function(y, x, a = seq(0.1, 1, by = 0.1), nfolds = 10, folds = N
     abline(v = a, col = "lightgrey", lty = 2)
     abline(h = seq(min(kl), max(kl), length = 10), col = "lightgrey", lty = 2)
   }
-
   list(runtime = apa, kula = kula, kl = kl, opt = opt, value = per)
-}
+}
\ No newline at end of file
