@@ -19,8 +19,7 @@ glmpcr.tune <- function(y, x, nfolds = 10, maxk = 10, folds = NULL, ncores = 1,
   n <- dim(x)[1]
   p <- dim(x)[2]
   if ( maxk > p ) maxk <- p  ## just a check
-  if ( is.null(folds) )  folds <- Compositional::makefolds(y, nfolds = nfolds,
-                                                           stratified = FALSE, seed = seed)
+  if ( is.null(folds) )  folds <- Compositional::makefolds(y, nfolds = nfolds, stratified = FALSE, seed = seed)
   nfolds <- length(folds)
   msp <- matrix( nrow = nfolds, ncol = maxk )
   ## deigma will contain the positions of the test set
@@ -64,21 +63,25 @@ glmpcr.tune <- function(y, x, nfolds = 10, maxk = 10, folds = NULL, ncores = 1,
 
   } else {
     runtime <- proc.time()
-    requireNamespace("doParallel", quietly = TRUE, warn.conflicts = FALSE)
-    cl <- parallel::makePSOCKcluster(ncores)
-    doParallel::registerDoParallel(cl)
-    er <- numeric(maxk)
-    if ( is.null(folds) )  folds <- Compositional::makefolds(y, nfolds = nfolds,
-                                                             stratified = FALSE, seed = seed)
-    msp <- foreach::foreach(vim = 1:nfolds, .combine = rbind, .packages = "Rfast", .export = c("glm_logistic", "glm_poisson") ) %dopar% {
+    
+    cl <- parallel::makeCluster(ncores)
+    # Load required packages on workers
+    parallel::clusterEvalQ(cl, library(Rfast))
+    # Export only what workers need
+    parallel::clusterExport(cl, 
+                           varlist = c("y", "x", "folds", "maxk", "oiko"), 
+                           envir = environment())
+    
+    msp <- t( parallel::parSapply(cl, 1:nfolds, function(vim) {
+      er <- numeric(maxk)
       ytest <- y[ folds[[ vim ]] ]  ## test set dependent vars
       ytrain <-  y[ -folds[[ vim ]] ]   ## train set dependent vars
       xtrain <- x[ -folds[[ vim ]], , drop = FALSE]   ## train set independent vars
       xtest <- x[ folds[[ vim ]], , drop = FALSE]  ## test set independent vars
-	  vec <- prcomp(xtrain, center = FALSE)$rotation
+	    vec <- prcomp(xtrain, center = FALSE)$rotation
       z <- xtrain %*% vec  ## PCA scores
 
-      for ( j in 1:maxk) {
+      for ( j in 1:maxk ) {
         if (oiko == "binomial") {
           be <- Rfast::glm_logistic(z[, 1:j], ytrain)$be
         } else {
@@ -87,7 +90,7 @@ glmpcr.tune <- function(y, x, nfolds = 10, maxk = 10, folds = NULL, ncores = 1,
         ztest <- xtest %*% vec[, 1:j, drop = FALSE]  ## PCA scores
         es <- as.vector( ztest %*% be[-1] ) + be[1]
 
-        if (oiko == "binomial") {
+        if ( oiko == "binomial" ) {
           est <- exp(es) / ( 1 + exp(es) )
           ri <-  -2 *( ytest * log(est) + (1 - ytest) * log(1 - est) )
         } else {
@@ -96,9 +99,10 @@ glmpcr.tune <- function(y, x, nfolds = 10, maxk = 10, folds = NULL, ncores = 1,
         }
         er[j] <- sum( ri, na.rm = TRUE )
       }
-      return(er)
-    }
-    stopCluster(cl)
+      er
+    }))
+    
+    parallel::stopCluster(cl)
     runtime <- proc.time() - runtime
   }
 
@@ -113,4 +117,4 @@ glmpcr.tune <- function(y, x, nfolds = 10, maxk = 10, folds = NULL, ncores = 1,
   performance <- min(mpd)
   names(performance) <- "MPD"
   list(msp = msp, mpd = mpd, k = which.min(mpd), performance = performance, runtime = runtime)
-}
+}
\ No newline at end of file
