@@ -21,7 +21,6 @@ js.compreg <- function(y, x, con = TRUE, B = 1, ncores = 1, xnew = NULL) {
     M <- ( mu + y ) / 2
     sum( - y * log1p(mu / y) + mu * log(mu / M), na.rm = TRUE )
   }
-
   n <- dim(y)[1]  ## sample size
   x <- model.matrix(y ~ ., data.frame(x) )
   if ( !con )  x <- x[, -1, drop = FALSE]
@@ -32,7 +31,6 @@ js.compreg <- function(y, x, con = TRUE, B = 1, ncores = 1, xnew = NULL) {
   if ( is.null( namy ) )  {
     namy <- paste("Y", 2:(d + 1), sep = "")
   } else namy <- namy[-1]
-
   ## the next lines minimize the kl.compreg function and obtain the estimated betas
   runtime <- proc.time()
   ini <- as.vector( t( Compositional::kl.compreg(y, x[, -1, drop = FALSE], con = con)$be ) )
@@ -44,53 +42,52 @@ js.compreg <- function(y, x, con = TRUE, B = 1, ncores = 1, xnew = NULL) {
   be <- matrix(qa$estimate, byrow = TRUE, ncol = d)
   covb <- NULL
   runtime <- proc.time() - runtime
-
   if (B > 1) {
-  betaboot <- matrix( nrow = B, ncol = length(ini) )
-  nc <- ncores
-    if (nc <= 1) {
+    betaboot <- matrix( nrow = B, ncol = length(ini) )
+    if (ncores <= 1) {
       runtime <- proc.time()
       for (i in 1:B) {
 	      ida <- Rfast2::Sample.int(n, n, replace = TRUE)
         yb <- y[ida, ]
         xb <- x[ida, ]
-        ini <- as.vector( t( Compositional::kl.compreg(yb, xb[, -1, drop = FALSE], con = con)$be ) ) ## initial values
-        qa <- nlm(jsreg, ini, y = yb, x = xb, d = d)
+        qa <- nlm(jsreg, be, y = yb, x = xb, d = d)
         qa <- nlm(jsreg, qa$estimate, y = yb, x = xb, d = d)
         qa <- nlm(jsreg, qa$estimate, y = yb, x = xb, d = d)
         betaboot[i, ] <- qa$estimate
       }  ##  end for (i in 1:B) {
       covb <- cov(betaboot)
       runtime <- proc.time() - runtime
-
     } else {
       runtime <- proc.time()
-      requireNamespace("doParallel", quietly = TRUE, warn.conflicts = FALSE)
-      cl <- parallel::makePSOCKcluster(ncores)
-      doParallel::registerDoParallel(cl)
-      betaboot <- foreach::foreach(i = 1:B, .combine = rbind, .packages = "Rfast2",
-	    .export = c("Sample.int", "jsreg", "kl.compreg") ) %dopar% {
-		  ida <- Rfast2::Sample.int(n, n, replace = TRUE)
+      
+      cl <- parallel::makeCluster(ncores)
+      # Load required packages on workers
+      parallel::clusterEvalQ(cl, library(Rfast2))
+      # Export only what workers need
+      parallel::clusterExport(cl, 
+                             varlist = c("jsreg", "y", "x", "n", "d", "be"), 
+                             envir = environment())
+      
+      betaboot <- t( parallel::parSapply(cl, 1:B, function(i) {
+        ida <- Rfast2::Sample.int(n, n, replace = TRUE)
         yb <- y[ida, ]
         xb <- x[ida, ]
         suppressWarnings({
-          ini <- as.vector( t( Compositional::kl.compreg(yb, xb[, -1, drop = FALSE], con = con)$be ) ) ## initial values
-          qa <- nlm(jsreg, ini, y = yb, x = xb, d = d)
+          qa <- nlm(jsreg, be, y = yb, x = xb, d = d)
           qa <- nlm(jsreg, qa$estimate, y = yb, x = xb, d = d)
           qa <- nlm(jsreg, qa$estimate, y = yb, x = xb, d = d)
+          qa$estimate  
         })
-        return( qa$estimate )
-      }  ##  end foreach
+      })) 
+      
       parallel::stopCluster(cl)
       covb <- cov(betaboot)
       runtime <- proc.time() - runtime
     }  ##  end (nc <= 1) {
-
     nam <- NULL
     for (i in 1:p)  nam <- c(nam, paste(namy, ":", namx[i], sep = "") )
     colnames(covb) <- rownames(covb) <- nam
   }  ##  end if (B > 1) {
-
   if ( !is.null(xnew) ) {
     xnew <- model.matrix(~., data.frame(xnew) )
     if ( !con )  xnew <- xnew[, -1, drop = FALSE]
@@ -98,8 +95,7 @@ js.compreg <- function(y, x, con = TRUE, B = 1, ncores = 1, xnew = NULL) {
     est <- mu / Rfast::rowsums(mu)
     colnames(est) <- colnames(y)
   }  else  est <- NULL
-
   colnames(be) <- namy
   rownames(be) <- namx
   list(runtime = runtime, be = be, covbe = covb, est = est)
-}
+}
\ No newline at end of file
