@@ -75,9 +75,6 @@ kl.compreg <- function(y, x, con = TRUE, B = 1, ncores = 1, xnew = NULL, tol = 1
       }  ##  end  for (i in 1:B) {
 
     } else {
-      requireNamespace("doParallel", quietly = TRUE, warn.conflicts = FALSE)
-      cl <- parallel::makePSOCKcluster(ncores)
-      doParallel::registerDoParallel(cl)
       X <- model.matrix(y~., data.frame(x) )
       if ( !con )  X <- X[, -1, drop = FALSE]
       p <- dim(X)[2]
@@ -85,12 +82,24 @@ kl.compreg <- function(y, x, con = TRUE, B = 1, ncores = 1, xnew = NULL, tol = 1
       dm <- dim(Y)
       n <- dm[1]    ;   d <- dm[2]
       b1 <- mod$be
-      betaboot <- matrix( nrow = B, ncol = prod( dim(b1) ) )
       id <- matrix(1:c(p * d), ncol = d)
       der <- numeric(d * p)
       der2 <- matrix(0, p * d, p * d)
-      betaboot <- foreach::foreach( i = 1:B, .combine = rbind, .packages = c("Rfast", "Rfast2", "nnet"),
-	              .export = c("klcompreg.boot", "multinom", "Sample.int") ) %dopar% {
+      
+      cl <- parallel::makeCluster(ncores)
+      # Load required packages on workers
+      parallel::clusterEvalQ(cl, {
+        library(Rfast2)
+        library(Compositional)
+        library(nnet)
+      })
+      # Export only what workers need
+      parallel::clusterExport(cl, 
+                             varlist = c("Y", "X", "n", "p", "d", "b1", "id", 
+                                        "der", "der2", "tol", "maxiters"), 
+                             envir = environment())
+      
+      betaboot <- t( parallel::parSapply(cl, 1:B, function(i) {
         ida <- Rfast2::Sample.int(n, n, replace = TRUE)
         yb <- Y[ida, ]
         xb <- X[ida, ]
@@ -99,8 +108,9 @@ kl.compreg <- function(y, x, con = TRUE, B = 1, ncores = 1, xnew = NULL, tol = 1
           mod <- nnet::multinom(yb ~ xb, trace = FALSE)
           bb <- t( coef(mod) )
         }
-        return( as.vector(bb$be) )
-      }  ##  end  foreach
+        as.vector(bb$be)
+      })) 
+      
       parallel::stopCluster(cl)
     }  ##  end if (ncores <= 1) {
     covb <- cov(betaboot)
@@ -118,10 +128,4 @@ kl.compreg <- function(y, x, con = TRUE, B = 1, ncores = 1, xnew = NULL, tol = 1
     res <- list(runtime = runtime, iters = iters, loglik = loglik, be = be, covbe = covb, est = est)
   }
   res
-}
-
-
-
-
-
-
+}
\ No newline at end of file
