@@ -1,5 +1,5 @@
-rda.tune <- function(x, ina, nfolds = 10, gam = seq(0, 1, by = 0.1), del = seq(0, 1, by = 0.1),
-                     ncores = 1, folds = NULL, stratified = TRUE, seed = NULL) {
+rda.tune <- function( x, ina, nfolds = 10, gam = seq(0, 1, by = 0.1), del = seq(0, 1, by = 0.1),
+                      ncores = 1, folds = NULL, stratified = TRUE, seed = NULL ) {
   ## x contains the data
   ## gam is between pooled covariance and diagonal
   ## gam*Spooled+(1-gam)*diagonal
@@ -22,12 +22,20 @@ rda.tune <- function(x, ina, nfolds = 10, gam = seq(0, 1, by = 0.1), del = seq(0
   if (ncores > 1) {
     runtime <- proc.time()
     group <- matrix(nrow = length(gam), ncol = length(del) )
-    cl <- parallel::makePSOCKcluster(ncores)
-    doParallel::registerDoParallel(cl)
-    if ( is.null(folds) )  folds <- .makefolds(ina, nfolds = nfolds,
-                                                             stratified = stratified, seed = seed)
-    ww <- foreach::foreach(vim = 1:nfolds, .combine = cbind, .export = c("mahala", "rowMaxs", "colGroup", "cova"),
-	      .packages = c("Rfast", "Rfast2") ) %dopar% {
+
+    cl <- parallel::makeCluster(ncores)
+    # Load required packages on workers
+    parallel::clusterEvalQ(cl, {
+      library(Rfast)
+      library(Rfast2)
+    })
+    # Export only what workers need
+    parallel::clusterExport(cl, 
+                           varlist = c("x", "ina", "folds", "gam", "del", "D", "nc"), 
+                           envir = environment())
+    
+    ww <- parallel::parSapply(cl, 1:nfolds, function(vim) {
+      sk <- array( dim = c(D, D, nc) )
       test <- x[ folds[[ vim ]], , drop = FALSE]  ## test sample
       id <- ina[ folds[[ vim ]] ] ## groups of test sample
       train <- x[ -folds[[ vim ]], , drop = FALSE]   ## training sample
@@ -42,21 +50,25 @@ rda.tune <- function(x, ina, nfolds = 10, gam = seq(0, 1, by = 0.1), del = seq(0
       Sp <- colSums( aperm(s) ) / (sum(na) - nc)  ## pooled covariance matrix
       sp <- diag( sum( diag( Sp ) ) / D, D )
       gr <- matrix(nrow = length( folds[[ vim ]] ), ncol = nc)
+      group <- matrix(nrow = length(gam), ncol = length(del) )
 
       for ( k1 in 1:length(gam) ) {
         Sa <- gam[k1] * Sp + (1 - gam[k1]) * sp  ## regularised covariance matrix
         for ( k2 in 1:length(del) ) {
           for (j in 1:nc) {
             Ska <- del[k2] * sk[, , j] + (1 - del[k2]) * Sa
-            gr[, j] <- ci[j] - log( det( Ska ) ) - Rfast::mahala( test, mesi[j, ], Ska )
+            mah <- try( Rfast::mahala( test, mesi[j, ], Ska ), silent = TRUE )
+            if ( identical(class(mah), "try-error") )  mah <- NA
+            gr[, j] <- ci[j] - log( det( Ska ) ) - mah
             ## the scores are doubled, for efficiency I did not multiply with 0.5
           }
           g <- Rfast::rowMaxs(gr)
           group[k1, k2] <- mean( g == id )
         }
       }
-      return( as.vector( group ) )
-    }
+      as.vector( group )
+    })
+    
     parallel::stopCluster(cl)
 
     per <- array( dim = c( lg, ld, nfolds ) )
@@ -89,7 +101,9 @@ rda.tune <- function(x, ina, nfolds = 10, gam = seq(0, 1, by = 0.1), del = seq(0
         for ( k2 in 1:length(del) ) {
           for (j in 1:nc) {
             Ska <- del[k2] * sk[, , j] + (1 - del[k2]) * Sa
-            gr[, j] <- ci[j] - log( det( Ska ) ) - Rfast::mahala( test, mesi[j, ], Ska )
+            mah <- try( Rfast::mahala( test, mesi[j, ], Ska ), silent = TRUE )
+            if ( identical(class(mah), "try-error") )  mah <- NA
+            gr[, j] <- ci[j] - log( det( Ska ) ) - mah
             ## the scores are doubled, for efficiency I did not multiply with 0.5
           }
           g <- Rfast::rowMaxs(gr)
@@ -104,7 +118,7 @@ rda.tune <- function(x, ina, nfolds = 10, gam = seq(0, 1, by = 0.1), del = seq(0
   percent <- t( colMeans( aperm(per) ) )
   su <- apply(per, 1:2, sd)
   dimnames(percent) <- dimnames(su) <- list(gamma = gam, delta = del)
-  confa <- as.vector( which(percent == max( percent ), arr.ind = TRUE )[1, ] )
+  confa <- as.vector( which(percent == max( percent, na.rm = TRUE ), arr.ind = TRUE )[1, ] )
   result <- cbind( max(percent), gam[ confa[1] ], del[ confa[2] ] )
   colnames(result) <- c('optimal', 'best gamma', 'best delta')
   list(per = per, percent = percent, se = su, result = result, runtime = runtime)
@@ -114,6 +128,7 @@ rda.tune <- function(x, ina, nfolds = 10, gam = seq(0, 1, by = 0.1), del = seq(0
 
 
 
+
 .makefolds <- function(ina, nfolds = 10, stratified = TRUE, seed = NULL) {
   names <- paste("Fold", 1:nfolds)
   runs <- sapply(names, function(x) NULL)
@@ -149,7 +164,4 @@ rda.tune <- function(x, ina, nfolds = 10, gam = seq(0, 1, by = 0.1), del = seq(0
 
   if ( length(runs[[ nfolds ]]) == 0 ) runs[[ nfolds ]] <- NULL
   runs
-}
-
-
-
+}
\ No newline at end of file
