@@ -20,7 +20,6 @@ symkl.compreg <- function(y, x, con = TRUE, B = 1, ncores = 1, xnew = NULL) {
     mu <- mu1 / rowSums(mu1)
     sum( (y - mu) * log(y / mu), na.rm = TRUE )
   }
-
   n <- dim(y)[1]  ## sample size
   x <- model.matrix(y ~ ., data.frame(x) )
   if ( !con )  x <- x[, -1, drop = FALSE]
@@ -31,7 +30,6 @@ symkl.compreg <- function(y, x, con = TRUE, B = 1, ncores = 1, xnew = NULL) {
   if ( is.null( namy ) )  {
     namy <- paste("Y", 2:(d + 1), sep = "")
   } else namy <- namy[-1]
-
   ## the next lines minimize the symkl function and obtain the estimated betas
   runtime <- proc.time()
   ini <- rnorm( d * dim(x)[2] )
@@ -43,7 +41,6 @@ symkl.compreg <- function(y, x, con = TRUE, B = 1, ncores = 1, xnew = NULL) {
   be <- matrix(qa$estimate, byrow = TRUE, ncol = d)
   covb <- NULL
   runtime <- proc.time() - runtime
-
   if (B > 1) {
   betaboot <- matrix( nrow = B, ncol = length(ini) )
   nc <- ncores
@@ -61,35 +58,37 @@ symkl.compreg <- function(y, x, con = TRUE, B = 1, ncores = 1, xnew = NULL) {
       }
       covb <- cov(betaboot)
       runtime <- proc.time() - runtime
-
     } else {
       runtime <- proc.time()
-      requireNamespace("doParallel", quietly = TRUE, warn.conflicts = FALSE)
-      cl <- parallel::makePSOCKcluster(ncores)
-      doParallel::registerDoParallel(cl)
-      betaboot <- foreach::foreach( i = 1:B, .combine = rbind, .packages = "Rfast2",
-	              .export = "symkl" ) %dopar% {
+      
+      cl <- parallel::makeCluster(ncores)
+      # Load required packages on workers
+      parallel::clusterEvalQ(cl, library(Rfast2))
+      # Export only what workers need
+      parallel::clusterExport(cl, 
+                             varlist = c("symkl", "y", "x", "n", "d"), 
+                             envir = environment())
+      
+      betaboot <- t( parallel::parSapply(cl, 1:B, function(i) {
         ida <- Rfast2::Sample.int(n, n, replace = TRUE)
-        yb <- y[ida, ]
-        xb <- x[ida, ]
+        yb <- y[ida, ]  ;  xb <- x[ida, ]
         ini <- rnorm( d * dim(x)[2] )
-		    suppressWarnings({
+        suppressWarnings({
           qa <- nlm(symkl, ini, y = yb, x = xb, d = d)
           qa <- nlm(symkl, qa$estimate, y = yb, x = xb, d = d)
           qa <- nlm(symkl, qa$estimate, y = yb, x = xb, d = d)
+          qa$estimate  
         })
-        return(qa$estimate)
-      }  ##  end foreach
+      })) 
+      
       parallel::stopCluster(cl)
       covb <- cov(betaboot)
       runtime <- proc.time() - runtime
     }  ##  end if (nc < 1)
-
   nam <- NULL
   for (i in 1:p)  nam <- c(nam, paste(namy, ":", namx[i], sep = "") )
   colnames(covb) <- rownames(covb) <- nam
   }  ##  end if (B > 1) {
-
   if ( !is.null(xnew) ) {
     xnew <- model.matrix(~., data.frame(xnew) )
     if ( !con )  xnew <- xnew[, -1, drop = FALSE]
@@ -97,8 +96,7 @@ symkl.compreg <- function(y, x, con = TRUE, B = 1, ncores = 1, xnew = NULL) {
     est <- mu / Rfast::rowsums(mu)
     colnames(est) <- colnames(y)
   }  else  est <- NULL
-
   colnames(be) <- namy
   rownames(be) <- namx
   list(runtime = runtime, be = be, covbe = covb, est = est)
-}
+}
\ No newline at end of file
