@@ -16,14 +16,12 @@ tv.compreg <- function(y, x, con = TRUE, B = 1, ncores = 1, xnew = NULL) {
   if ( is.null( namy ) )  {
     namy <- paste("Y", 2:(d + 1), sep = "")
   } else namy <- namy[-1]
-
   tvreg <- function(para, y, x, d) {
     be <- matrix(para, byrow = TRUE, ncol = d)
     mu1 <- cbind(1, exp(x%*%be) )
     mu <- mu1 / rowSums(mu1)
     sum( abs(y - mu) )  ## Total variation
   }
-
   ## the next lines minimize the kl.compreg function and obtain the estimated betas
   runtime <- proc.time()
   ini <- as.vector( t( Compositional::kl.compreg(y, x[, -1, drop = FALSE], con = con)$be ) )
@@ -35,7 +33,6 @@ tv.compreg <- function(y, x, con = TRUE, B = 1, ncores = 1, xnew = NULL) {
   be <- matrix(qa$estimate, byrow = TRUE, ncol = d)
   covb <- NULL
   runtime <- proc.time() - runtime
-
   if (B > 1) {
   betaboot <- matrix( nrow = B, ncol = length(ini) )
   nc <- ncores
@@ -45,43 +42,44 @@ tv.compreg <- function(y, x, con = TRUE, B = 1, ncores = 1, xnew = NULL) {
         ida <- Rfast2::Sample.int(n, n, replace = TRUE)
         yb <- y[ida, ]
         xb <- x[ida, ]
-        ini <- as.vector( t( Compositional::kl.compreg(yb, xb[, -1, drop = FALSE], con = con)$be ) ) ## initial values
-        qa <- nlm(tvreg, ini, y = yb, x = xb, d = d)
+        qa <- nlm(tvreg, be, y = yb, x = xb, d = d)
         qa <- nlm(tvreg, qa$estimate, y = yb, x = xb, d = d)
         qa <- nlm(tvreg, qa$estimate, y = yb, x = xb, d = d)
         betaboot[i, ] <- qa$estimate
       }  ##  end for (i in 1:B) {
       covb <- cov(betaboot)
       runtime <- proc.time() - runtime
-
     } else {
       runtime <- proc.time()
-      requireNamespace("doParallel", quietly = TRUE, warn.conflicts = FALSE)
-      cl <- parallel::makePSOCKcluster(ncores)
-      doParallel::registerDoParallel(cl)
-      betaboot <- foreach::foreach(i = 1:B, .combine = rbind, .packages = "Rfast2",
-	              .export=c("Sample.int", "tvreg", "kl.compreg") ) %dopar% {
+      
+      cl <- parallel::makeCluster(ncores)
+      # Load required packages on workers
+      parallel::clusterEvalQ(cl, library(Rfast2))
+      # Export only what workers need
+      parallel::clusterExport(cl, 
+                             varlist = c("tvreg", "y", "x", "n", "d", "be"), 
+                             envir = environment())
+      
+      betaboot <- t( parallel::parSapply(cl, 1:B, function(i) {
         ida <- Rfast2::Sample.int(n, n, replace = TRUE)
         yb <- y[ida, ]
         xb <- x[ida, ]
-        ini <- as.vector( t( Compositional::kl.compreg(yb, xb[, -1, drop = FALSE], con = con)$be ) ) ## initial values
         suppressWarnings({
-          qa <- nlm(tvreg, ini, y = yb, x = xb, d = d)
+          qa <- nlm(tvreg, be, y = yb, x = xb, d = d)
           qa <- nlm(tvreg, qa$estimate, y = yb, x = xb, d = d)
           qa <- nlm(tvreg, qa$estimate, y = yb, x = xb, d = d)
+          qa$estimate  
         })
-        return( qa$estimate )
-      }  ##  end foreach
+      })) 
+      
       parallel::stopCluster(cl)
       covb <- cov(betaboot)
       runtime <- proc.time() - runtime
     }  ##  end (nc <= 1) {
-
     nam <- NULL
     for (i in 1:p)  nam <- c(nam, paste(namy, ":", namx[i], sep = "") )
     colnames(covb) <- rownames(covb) <- nam
   }  ##  end if (B > 1) {
-
   if ( !is.null(xnew) ) {
     xnew <- model.matrix(~., data.frame(xnew) )
     if ( !con )  xnew <- xnew[, -1, drop = FALSE]
@@ -89,8 +87,7 @@ tv.compreg <- function(y, x, con = TRUE, B = 1, ncores = 1, xnew = NULL) {
     est <- mu / Rfast::rowsums(mu)
     colnames(est) <- colnames(y)
   }  else  est <- NULL
-
   colnames(be) <- namy
   rownames(be) <- namx
   list(runtime = runtime, be = be, covbe = covb, est = est)
-}
+}
\ No newline at end of file
