@@ -1,4 +1,4 @@
-zadr <- function(y, x, con = TRUE, B = 1, ncores = 2, xnew = NULL) {
+zadr <- function(y, x, con = TRUE, xnew = NULL) {
   ## y is the compositional data
   ## x is the independent variable(s)
   dm <- dim(y)
@@ -47,72 +47,17 @@ zadr <- function(y, x, con = TRUE, B = 1, ncores = 2, xnew = NULL) {
   phi <- exp( qa$par[1] )  ## final phi value
   be <- matrix( qa$par[-1], ncol = d ) ## final beta values
 
-  if ( B > 1 ) {
-    runtime <- proc.time()
-    requireNamespace("doParallel", quietly = TRUE, warn.conflicts = FALSE)
-    cl <- parallel::makePSOCKcluster(ncores)
-    doParallel::registerDoParallel(cl)
-    betaboot <- foreach::foreach( i = 1:B, .combine = rbind, .export = c("Sample.int", ".mixreg", "diri.nr"),
-                                 .packages = c("Rfast2", "Compositional") ) %dopar% {
-      ida <- Rfast2::Sample.int(n, n, replace = TRUE)
-      yb <- y[ida, ]
-      xb <- x[ida, ]
-
-      a1 <- which( Rfast::rowsums( yb > 0 ) == D )
-      a2 <- which( Rfast::rowsums( yb > 0 ) != D )
-      n1 <- length(a1)
-      n2 <- n - n1
-      ## n1 is the sample size of the compositional vectors with no zeros
-      ## n2 is the sample size of the compositional vectors with zeros
-      za <- yb[a2, , drop = FALSE]
-      za[za == 0] <- 1
-      za[ za < 1 ] <- 0
-      theta <- table( apply(za, 1, paste, collapse = ",") )
-      theta <- as.vector(theta)
-      const <- n1 * log(n1/n) + sum( theta * log(theta/n) )
-
-      y1 <- yb[a1, , drop = FALSE]
-      ly1 <- log( y1 )
-      x1 <- xb[a1, , drop = FALSE]
-      ly2 <- log( y[a2, , drop = FALSE] )
-      x2 <- xb[a2, , drop = FALSE]
-      n1 <- nrow(y1)    ;    n2 <- n - n1
-      ##############
-      beta.ini <- try( Compositional::kl.compreg(yb, xini[ida, ], con = con)$be, silent = TRUE )
-      if ( identical(class(beta.ini), "try-error") ) {
-        beta.ini <- be
-      }
-      ini.phi <- Compositional::zad.est(yb)$phi
-      ini.par <- c( log(ini.phi), as.vector(beta.ini) )  ## initial parameter values
-      z <- list(ly1 = ly1, ly2 = ly2, x1 = x1, x2 = x2, a1 = a1, a2 = a2)
-      suppressWarnings({
-        qa <- optim( ini.par, .mixreg, z = z )
-        qa <- optim( qa$par, .mixreg, z = z )
-        qa <- optim( qa$par, .mixreg, z = z, control = list(maxit = 1000) )
-      })
-      return( qa$par )
-    }  ##  end foreach
-    parallel::stopCluster(cl)
-    sigma <- cov(betaboot)
+  sigma <- try( solve( qa$hessian ), silent = TRUE )
+  if ( !identical( class(sigma), "try-error") ) {
     seb <- sqrt( diag(sigma) )
     seb <- matrix(seb[-1], ncol = d)
     colnames(seb) <- colnames( y[, -1] )
     rownames(seb) <- colnames(x)
-    runtime <- proc.time() - runtime
-
   } else {
-    sigma <- try( solve( qa$hessian ), silent = TRUE )
-    if ( !identical( class(sigma), "try-error") ) {
-      seb <- sqrt( diag(sigma) )
-      seb <- matrix(seb[-1], ncol = d)
-      colnames(seb) <- colnames( y[, -1] )
-      rownames(seb) <- colnames(x)
-    } else {
-      sigma <- NULL
-      seb <- NULL
-    }
-  }  ##  end if (B > 1)
-
+    sigma <- NULL
+    seb <- NULL
+  }
+  
   colnames(be) <- colnames( y[, -1] )
   rownames(be) <- colnames(x)
   est <- NULL
